@@ -80,13 +80,8 @@ void DeletePointer(std::shared_ptr<T>* ptr) {
 template <typename T>
 using Pointer = cpp11::external_pointer<std::shared_ptr<T>, DeletePointer<T>>;
 
-#if ARROW_VERSION_MAJOR >= 18
 using ChunkResolver = arrow::ChunkResolver;
 using ChunkLocation = arrow::ChunkLocation;
-#else
-using ChunkResolver = arrow::internal::ChunkResolver;
-using ChunkLocation = arrow::internal::ChunkLocation;
-#endif
 
 class ArrowAltrepData {
  public:
@@ -577,7 +572,7 @@ struct AltrepFactor : public AltrepVectorBase<AltrepFactor> {
 
   static SEXP Duplicate(SEXP alt, Rboolean /* deep */) {
     // the representation integer vector
-    SEXP dup = PROTECT(Rf_lazy_duplicate(Materialize(alt)));
+    SEXP dup = PROTECT(Rf_shallow_duplicate(Materialize(alt)));
 
     // additional attributes from the altrep
     SEXP atts = PROTECT(Rf_duplicate(ATTRIB(alt)));
